<?php

namespace Drupal\google_stt\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Google\Cloud\Speech\V1\RecognitionAudio;
use Google\Cloud\Speech\V1\RecognitionConfig;
use Google\Cloud\Speech\V1\RecognitionConfig\AudioEncoding;
use Google\Cloud\Speech\V1\SpeechClient;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Google\Cloud\Storage\StorageClient;

/**
 * Class SpeechToTextForm.
 */
class SpeechToTextForm extends FormBase {

  /**
   * Drupal\Core\File\FileSystemInterface definition.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  private $uri;

  protected $entityTypeManager;

  /**
   * Constructs a new SpeechToTextForm object.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   */
  public function __construct(
    FileSystemInterface $file_system,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    $this->fileSystem = $file_system;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_system'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'speech_to_text_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Check if Google Service Account credentials json file is present.
    if (!file_exists('sites/default/service-account.json')) {
      drupal_set_message($this->t('The Google API Credentials file is missingg. ' .
        'Go <a href="https://console.cloud.google.com/apis/credentials" target="_blank">here</a>' .
        ' and do that. More information <a href="https://cloud.google.com/speech-to-text/docs/quickstart" target="_blank">here</a> on how to do that.'), 'warning');
    }
    $form['language_code'] = [
      '#type' => 'select',
      '#title' => 'Language Code',
      '#options' => [
        'en-US' => 'en-US',
        'en-UK' => 'en-UK',
        'en-IN' => 'en-IN',
        ],
      '#description' => 'Choose the English language variant.',
      '#default_value' => 'en-US',
      '#required' => TRUE,
    ];

    $form['bucket_name'] = [
      '#type' => 'textfield',
      '#title' => 'Googele Storage Bucket Name',
      '#description' => 'Your google storage bucket name. If you don\'t have one' .
        'create a google storage bucket by signing into your Google Developer Console.' .
        'Example : gs://your-google-bucket (Add the value "your-google-bucket" here.)',
      '#required' => TRUE,
    ];

    $form['upload_file'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Upload File'),
      '#description' => $this->t('Upload the Audio/Video file to be transcribed. Currently supported file types: .wav, .flac, .m4a, .mp4, .mp3'),
      '#required' => TRUE,
      '#upload_validators' => [
        'file_validate_extensions' => ['mp3 mp4 m4a flac wav mpeg ogg m4p m4v avi wmv mov'],
        'file_validate_size' => [1024 * 1024 * 1024],
      ],
      '#upload_location' => 'public://audio_files',
    ];

    $form['actions'] = [
      '#type' => 'button',
      '#value' => $this->t('Transcribe'),
      '#ajax' => [
        'callback' => '::transcribeFile',
        'wrapper' => 'transcribe-output',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Transcribing...'),
        ]
      ]
    ];
    $form['#suffix'] = '<div id="transcribe-output"></div>';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validate(array &$form, FormStateInterface $form_state) {
    $fid = reset($form_state->getValue('upload_file'));
    $file = $this->entityTypeManager->getStorage('file')->load($fid);
    $uri = $file->getFileUri();
    $fname = $file->getFilename();

    $new_name = str_replace(' ', '', $fname);

    $new_uri = str_replace($fname, $new_name, $uri);

    // And finally rename the file.
    file_move($file, $new_uri);
  }

  public function transcribeFile(array &$form, FormStateInterface $form_state) {
    $this->validate($form, $form_state);
    putenv("GOOGLE_APPLICATION_CREDENTIALS=sites/default/service-account.json");
    // Display result.
    $fid = reset($form_state->getValue('upload_file'));
    $file = $this->entityTypeManager->getStorage('file')->load($fid);
    $real_path = $this->fileSystem->realpath($file->getFileUri());
    $name = explode('.', $file->getFilename())[0];
    $bucket = $form_state->getValue('bucket_name');
    $this->uri = 'gs://' . $bucket . '/' . $name . '.wav';
    $output_file = $this->convertVideo($real_path, $file->getFilename());
    $this->uploadFile($output_file, $bucket);

    list($config, $audio) = $this->setAudioDeets($output_file, $form_state->getValue('language_code'));
    # Instantiates a client
    $transcript = $this->recognizeOperation($config, $audio);

    //shell_exec("rm $output_file");

    $response = new AjaxResponse();
    $response->addCommand(new HtmlCommand('#transcribe-output', "<h3>Transcript : </h3> $transcript"));
    return $response;
  }

  public function uploadFile($output_file, $bucket) {
    $storage = new StorageClient();
    $bucket = $storage->bucket($bucket);
    // Upload a file to the bucket.
    $bucket->upload(
      fopen($output_file, 'r')
    );
  }

  public function convertVideo($real_path, $file_name) {
    $name = explode('.', $file_name)[0];
    $output_file = $this->fileSystem->realpath(file_default_scheme() . "://audio_files/$name.wav");
    shell_exec("ffmpeg -y -i $real_path -ac 1 $output_file");
    return $output_file;
  }

  public function setAudioDeets($output_file, $languageCode) {
    $audioDeets = json_decode(shell_exec("ffprobe -v quiet -print_format json -show_format -show_streams $output_file"), TRUE);
    $audio = (new RecognitionAudio())
      ->setUri($this->uri);
    // set config
    $config = (new RecognitionConfig())
      ->setEncoding(AudioEncoding::LINEAR16)
      ->setLanguageCode((string) $languageCode)
      ->setAudioChannelCount(1)
      ->setSampleRateHertz((int) $audioDeets['streams'][0]['sample_rate']);

    return [$config, $audio];
  }

  public function recognizeOperation($config, $audio) {
    $client = new SpeechClient();
    // create the asynchronous recognize operation
    $operation = $client->longRunningRecognize($config, $audio);
    $operation->pollUntilComplete();
    $output_transcript = '';
    if ($operation->operationSucceeded()) {
      $response = $operation->getResult();

      // each result is for a consecutive portion of the audio. iterate
      // through them to get the transcripts for the entire audio file.
      foreach ($response->getResults() as $result) {
        $alternatives = $result->getAlternatives();
        $mostLikely = $alternatives[0];
        $transcript = $mostLikely->getTranscript();
        $output_transcript .= $transcript;
      }
    } else {
      $output_transcript = $operation->getError()->getMessage();
    }
    $client->close();
    return $output_transcript;
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // TODO: Implement submitForm() method.
  }

}
